/* Sentinel - IRC Statistical and Operator Services
** sqldump.c - MySQL dump equivalent to the datadump.
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** This file copyrighted to Ilya Shtift (gate) <ishtift@tagil.svrw.ru>
**
** $Id: sqldump.c,v 1.3 2003/05/12 00:53:53 wcampbel Exp $
*/

#include "setup.h"
#ifdef HAVE_MYSQL
#include <mysql.h>
#endif
#include "stats.h"
#include "struct.h"
#include "proto.h"

#define TABLE_SERVERS "server"
#define TABLE_NETINFO "netinfo"
#define TABLE_DAILY "daily"

#ifdef HAVE_MYSQL
MYSQL *myptr;
#endif

int db_query(const char *fmt, ...)
{
  va_list args;
  char buf[2048]; /* XXX define */
  va_start(args, fmt);
  vsnprintf(buf, 2048, fmt, args);
#ifdef HAVE_MYSQL
  if (mysql_real_query(myptr, buf, strlen(buf)))
  {
    log(L_CRIT, "Query failed: %s", mysql_error(myptr));
    return 1;
  }
#endif
  return 0;
}

void sqldump(char *MysqlServer, char *MysqlUser, char *MysqlPassword, char *MysqlDatabase)
{
#ifdef HAVE_MYSQL
  struct Server *s;
  int i;

  if (myptr)
    mysql_close(myptr);

  myptr = mysql_init(NULL);
  if (!mysql_real_connect(myptr, MysqlServer, MysqlUser, MysqlPassword, MysqlDatabase, 0, NULL, 0))
  {
    log(L_CRIT, "Failed to connect to database : %s", mysql_error(myptr));
    return;
  }
  db_query("DELETE FROM " TABLE_NETINFO);
  db_query("DELETE FROM " TABLE_DAILY);
  db_query("DELETE FROM " TABLE_SERVERS);

  db_query("INSERT INTO " TABLE_NETINFO " VALUES "
           "('%u', '%ld', '%ld', FROM_UNIXTIME(%ld), '%d', '%d', "
           "FROM_UNIXTIME(%ld), '%d', '%d', FROM_UNIXTIME(%ld),"
	   "'%d', '%d', FROM_UNIXTIME(%ld), '%d', FROM_UNIXTIME(%ld), '%ld', "
           "'%ld', FROM_UNIXTIME(%ld), '%d')",
          netinfo.away, netinfo.users, netinfo.maxusers,
          (long)netinfo.t_maxusers, netinfo.opers, netinfo.maxopers,
          (long)netinfo.t_maxopers, netinfo.servers, netinfo.maxservers,
          (long)netinfo.t_maxservers,
#ifdef IRCD_2_10
          netinfo.services, netinfo.maxservices,
          (long)netinfo.t_maxservices, netinfo.maxaway,
          (long)netinfo.t_maxaway,
#elif defined(IRCD_HYB7) || defined(IRCD_HYBSVS) || defined(IRCD_BAHAMUT)
          0, 0, 0L, netinfo.maxaway, (long)netinfo.t_maxaway,
#else
          0, 0, 0L, 0, 0L,
#endif
          netinfo.channels, netinfo.maxchannels,
          (long)netinfo.t_maxchannels, netinfo.invis);

  db_query("INSERT INTO " TABLE_DAILY " VALUES "
           "(NOW(), '%ld', FROM_UNIXTIME(%ld), '%d', FROM_UNIXTIME(%ld), '%d', "
           "FROM_UNIXTIME(%ld), '%d', FROM_UNIXTIME(%ld), '%ld', "
           "FROM_UNIXTIME(%ld))",
          daily.users, (long)daily.t_users, daily.opers,
          (long)daily.t_opers, daily.servers, (long)daily.t_servers,
#ifdef IRCD_2_10
          daily.services, (long)daily.t_services,
#else
          0, 0L,
#endif
          daily.channels, (long)daily.t_channels);

  for (i = 0; i < S_TABLE_SIZE; i++)
  {
    for (s = serverlist[i]; s; s = s->next)
    {
      db_query("INSERT INTO " TABLE_SERVERS " VALUES "
      "('%s', '%d', '%ld', '%ld', FROM_UNIXTIME(%ld), '%d', '%d', "
      "FROM_UNIXTIME(%ld), '%d', '%u', '%u', '%d', '%d', FROM_UNIXTIME(%ld), "
      "'%d', '%d', FROM_UNIXTIME(%ld), '%ld')",
        s->name, IsSHide(s) ? 1 : 0, s->users,
	s->stats->maxusers, (long)s->stats->t_maxusers,
        s->opers, s->stats->maxopers,
        (long)s->stats->t_maxopers, s->stats->numsplits,
        s->stats->operkills, s->stats->serverkills,
#ifdef IRCD_2_10
        s->services, s->stats->maxservices,
        (long)s->stats->t_maxservices, s->away,
        s->stats->maxaway, (long)s->stats->t_maxaway, s->invis);
#elif defined(IRCD_HYB7) || defined(IRCD_BAHAMUT) || defined(IRCD_HYBSVS)
        0, 0, 0L, s->away, s->stats->maxaway,
        (long)s->stats->t_maxaway, s->invis);
#else
        0, 0, 0L, 0, 0, 0L, s->invis);
#endif
    }
  }

  mysql_close(myptr);
  myptr = NULL;
#endif
}
